/**
 * $Id: DefaultFormAuthenticationFilter.java,v 1.0 18/5/16 上午11:13 chenmin Exp $
 * <p>
 */
package com.github.artislong.shiro;

import lombok.Data;
import lombok.experimental.Accessors;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.web.filter.authc.FormAuthenticationFilter;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import java.io.IOException;

/**
 * 为解决前后台分离架构的项目下，未登录时访问系统的跳转及对应的提示信息
 * Shiro原有逻辑为未登录则跳转到登录Url，在前后台分离架构下，此种方式显然不能满足要求，
 * 只能通过将请求转发到一个新的Url，给出未登录提示信息，由前台去控制路由跳转到登录页面
 * @author chenmin
 * @version $Id: DefaultFormAuthenticationFilter.java,v 1.1 18/5/16 上午11:13 chenmin Exp $
 * Created on 18/5/16 上午11:13
 */
@Slf4j
@Data
@Accessors(chain = true)
public class DefaultFormAuthenticationFilter extends FormAuthenticationFilter {

    public static final String NO_ACCESS_URL = "noAccessUrl";

    /**
     * 设置未登录请求跳转地址
     */
    private String noAccessUrl;

    @Override
    protected void redirectToLogin(ServletRequest request, ServletResponse response) throws IOException {
        String noAccessUrl = getNoAccessUrl();
        try {
            request.getRequestDispatcher(noAccessUrl).forward(request, response);
        } catch (ServletException e) {
            e.getMessage();
        }
    }
}