/**
 * $Id: SecurityUtil.java,v 1.0 2019-01-21 14:27 chenmin Exp $
 * <p>
 */
package com.github.artislong.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.subject.PrincipalCollection;
import org.apache.shiro.subject.SimplePrincipalCollection;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.BeanUtils;
import org.springframework.util.ObjectUtils;

import java.lang.reflect.ParameterizedType;

/**
 * @author chenmin
 * @version $Id: SecurityUtil.java,v 1.1 2019-01-21 14:27 chenmin Exp $
 * Created on 2019-01-21 14:27
 */
@Slf4j
public class SecurityUtils extends org.apache.shiro.SecurityUtils {

    /**
     * 通用登录
     * @param userName
     * @param password
     * @return
     * @throws RuntimeException
     */
    public static Subject login(String userName, String password) throws RuntimeException {
        Subject subject = getSubject();
        UsernamePasswordToken token = new UsernamePasswordToken(userName, password);
        // 登录
        subject.login(token);
        if (subject.isAuthenticated()) {
            return subject;
        }
        return null;
    }

    /**
     * 获取当前登录人员信息
     * @param <T>
     * @return
     */
    public static <T> T getCurrentUser() {
        Object principal = getSubject().getPrincipal();
        if (!ObjectUtils.isEmpty(principal)) {
            return (T) principal;
        }
        return null;
    }

    /**
     * 更新当前登录人员信息
     * @param newUser
     * @param <T>
     * @return
     */
    public static <T> T updateSubject(T newUser) {
        Subject subject = getSubject();
        T oldUser = getCurrentUser();
        BeanUtils.copyProperties(newUser, oldUser);
        PrincipalCollection principalCollection = subject.getPrincipals();
        String realmName = principalCollection.getRealmNames().iterator().next();
        PrincipalCollection newPrincipalCollection = new SimplePrincipalCollection(oldUser, realmName);
        //重新加载Principal
        subject.runAs(newPrincipalCollection);
        return newUser;
    }
}